@@ -4,20 +4,19 @@ use warnings;
 use strict;
 use Carp;
 use Data::Dump qw( dump );
-use MRO::Compat;
-use mro "c3";
+use Class::C3;
 use base qw( Class::Accessor::Fast );
 use JSON::XS ();
 use Scalar::Util qw( blessed );
 use CatalystX::CRUD::YUI::Serializer;
 
-our $VERSION = '0.030_01';
+our $VERSION = '0.008';
 
 __PACKAGE__->mk_accessors(
-    qw( yui results controller form c
+    qw( yui results controller form
         method_name pk columns show_related_values
         col_filter text_columns col_names url count counter
-        sort_by show_remove_button
+        field_names sort_by show_remove_button
         serializer_class serializer
         hide_pk_columns sort_dir title excel_url
         )
@@ -35,7 +34,6 @@ CatalystX::CRUD::YUI::LiveGrid - ExtJS LiveGrid objects
             form        => $form,
             method_name => $rel_info->{method},
             col_names   => $form->metadata->field_methods,
-            c           => $c,          # Catalyst context object
  );
   
  $livegrid->serialize;  # returns serialized results
@@ -43,15 +41,18 @@ CatalystX::CRUD::YUI::LiveGrid - ExtJS LiveGrid objects
 
 =head1 DESCRIPTION
 
-This class represents the data necessary to support
-the ExtJS-base LiveGrid component http://www.siteartwork.de/livegrid/.
+This is a subclass of CatalystX::CRUD::YUI::DataTable for use
+with the ExtJS-base LiveGrid component http://www.siteartwork.de/livegrid/.
+
+Only deviations from the CatalystX::CRUD::YUI::DataTable docs
+are described here.
 
 =head1 METHODS
 
 =head2 new( I<opts> )
 
-Create a YUI LiveGrid object.
-You usually call this via CatalystX::CRUD::YUI->livegrid( I<opts> ).
+Create a YUI DataTable object.
+You usually call this via CatalystX::CRUD::YUI->datatable( I<opts> ).
 
 I<opts> should include:
 
@@ -87,7 +88,7 @@ the objects being serialized, i.e., the controller governing I<form>.
 
 =back
 
-The new LiveGrid has the following accessors available:
+The new DataTable has the following accessors available:
 
 =over
 
@@ -97,7 +98,7 @@ The primary key of the table that I<results> represents.
 
 =item columns
 
-An arrayref of column hashrefs. YUI LiveGrid API requires these.
+An arrayref of column hashrefs. YUI DataTable API requires these.
 
 =item url
 
@@ -151,12 +152,18 @@ sub _init {
     my $controller = $self->{controller}
         or croak "controller required";
     my $form = $self->{form} or croak "form required";
-    my $app = ( $self->{c} || $form->app ) or croak "\$c object required";
+
+    my $app = $form->app || $form->app_class;
+    if ( !$app ) {
+
+        #carp dump $form;
+        croak "cannot get app from form->app or form->app_class";
+    }
 
     $self->{hide_pk_columns} = $controller->hide_pk_columns;
 
     # may be undef. this is the method we call on the the parent object,
-    # where parent $results isa RDBO and we are creating a livegrid out
+    # where parent $results isa RDBO and we are creating a datatable out
     # of its related objects.
     my $method_name = $self->{method_name};
 
@@ -170,6 +177,7 @@ sub _init {
     $self->columns( [] );
     $self->show_related_values( {} );
     $self->col_filter( [] );
+    $self->sort_by( $form->metadata->default_sort_by || $self->pk->[0] );
 
     #carp "col_names for $results: " . dump $self->col_names;
 
@@ -186,8 +194,6 @@ sub _init {
                 $results->query->{plain_query} || {}
             )
         );
-
-        $self->sort_by( $form->metadata->default_sort_by || $self->pk->[0] );
     }
     else {
 
@@ -195,7 +201,8 @@ sub _init {
         #carp "controller isa " . $controller;
 
         if ( !$method_name ) {
-            croak "method_name required for CatalystX::CRUD::Object livegrid";
+            croak
+                "method_name required for CatalystX::CRUD::Object datatable";
         }
         $self->url(
             $app->uri_for(
@@ -205,9 +212,6 @@ sub _init {
                 )
             )
         );
-
-        $self->sort_by( $form->metadata->default_related_sort_by
-                || $self->pk->[0] );
     }
 
     $self->{url} .= '?' unless $self->{url} =~ m/\?/;
@@ -232,7 +236,6 @@ sub _init {
         push( @filtered_col_names, $field_name );
 
         my $isa_field = $form->field($field_name);
-        my $isa_chain = $field_name =~ m/\./;
         my $col_def   = {
             key => $field_name,
 
@@ -241,13 +244,10 @@ sub _init {
             ? $isa_field->label . ''
             : ( $form->metadata->labels->{$field_name} || $field_name ),
 
-            sortable => ( $isa_field || $isa_chain )
+            sortable => $isa_field
             ? JSON::XS::true()
             : JSON::XS::false(),
 
-            sort_prefix =>
-                ( $form->metadata->sort_prefix->{$field_name} || '' ),
-
             type => $isa_field
             ? $self->_get_col_type( $isa_field->class )
             : 'string',
@@ -263,8 +263,8 @@ sub _init {
             { dataIndex => $col_def->{key}, type => $col_def->{type} }
         );
 
-        if ( ( $isa_field and $col_def->{type} eq 'string' )
-            or $isa_chain )
+        if (    $isa_field
+            and $col_def->{type} eq 'string' )
         {
             push( @{ $self->{text_columns} }, $field_name );
         }
@@ -307,18 +307,23 @@ sub _init {
         ? JSON::XS::false()
         : JSON::XS::true(),
 
-        type => 'pk',
+        type => $form->field($pk_name)
+        ? $self->_get_col_type( $form->field($pk_name)->class )
+        : 'string',
 
         # per-column click
         url => $app->uri_for( $form->metadata->field_uri($pk_name) ),
     };
-    push( @{ $self->{columns} }, $col_def );
+    push(
+        @{ $self->{columns} }, $col_def
+
+    );
     push(
         @{ $self->{col_filter} },
         { dataIndex => $col_def->{key}, type => $col_def->{type} }
     );
 
-    push( @filtered_col_names, $self->pk );
+    push( @filtered_col_names, $pk_name );
 
     $self->col_names( \@filtered_col_names );
 
@@ -329,14 +334,7 @@ sub _init {
         $self->{sort_dir} = 'ASC';
     }
 
-    my $moniker;
-    my $object_class = $form->metadata->object_class;
-    if ( $object_class->can('moniker') ) {
-        $moniker = $object_class->moniker;
-    }
-    else {
-        $moniker = $object_class;
-    }
+    my $moniker = ref $self->controller;
     $moniker =~ s/^.+:://;
     $self->{title} = 'Results for ' . $moniker;
 
@@ -361,7 +359,7 @@ sub column {
 
 =head2 serialize 
 
-Returns LiveGrid as array ref of hash refs, suitable
+Returns DataTable as array ref of hash refs, suitable
 for conversion to JSON or other transport type.
 
 =cut
@@ -369,7 +367,7 @@ for conversion to JSON or other transport type.
 sub serialize {
     my $self = shift;
     my $serializer = $self->serializer || $self->serializer_class->new;
-    return $serializer->serialize_livegrid($self);
+    return $serializer->serialize_datatable($self);
 }
 
 sub _get_col_type {
@@ -408,11 +406,10 @@ sub json_reader_opts {
     my $self = shift;
 
     return {
-        restful         => JSON::XS::true(),
         root            => 'response.value.items',
         versionProperty => 'response.value.version',
         totalProperty   => 'response.value.total_count',
-        id              => join( ';;', @{ $self->pk } ),
+        id              => $self->pk->[0],                 # TODO multiples
         }
 
 }
@@ -451,27 +448,69 @@ Returns array ref suitable for JSON-ifying for LiveGrid JS.
 sub column_defs {
     my $self = shift;
     my @defs;
+    my $widths = $self->__calc_widths;
     for my $col ( @{ $self->columns } ) {
         my $def = {
-            header     => $col->{label},
-            align      => 'left',
-            sortable   => $col->{sortable},
-            sortPrefix => $col->{sort_prefix},
-            dataIndex  => $col->{key},
-            type       => $col->{type},
+            header    => $col->{label},
+            align     => 'left',                     # TODO optional
+            width     => $widths->{ $col->{key} },
+            sortable  => $col->{sortable},
+            dataIndex => $col->{key},
+            type      => $col->{type},
         };
-        if ( $def->{type} eq 'pk' && $self->{hide_pk_columns} ) {
-            $def->{hidden} = JSON::XS::true();
-            $def->{type}   = 'string';
-        }
-        if ( $col->{key} =~ m/\./ ) {
-            $def->{type} = 'string';
-        }
         push( @defs, $def );
     }
     return \@defs;
 }
 
+# TODO memoize this?
+
+sub __calc_widths {
+    my $self = shift;
+    my %w;
+    my $max       = 600;
+    my $def_width = int( $max / scalar( @{ $self->columns } ) );
+    my $total     = 0;
+    for my $col ( @{ $self->columns } ) {
+        my $width = $def_width;
+        if ( $col->{type} eq 'date' ) {
+            $width = 100;
+        }
+        elsif ( $col->{type} eq 'string' ) {
+            $width = 100 unless $def_width > 100;
+        }
+        elsif ( $col->{type} eq 'numeric' ) {
+            $width = 60;
+        }
+        elsif ( $col->{type} eq 'boolean' ) {
+            $width = 12;
+        }
+
+        $w{ $col->{key} } = $width;
+        $total += $width;
+    }
+
+    if ( $total < $max ) {
+        while ( $total < $max ) {
+            for my $n ( keys %w ) {
+                $w{$n}++;
+                $total++;
+            }
+        }
+    }
+
+    if ( $total >= $max ) {
+        while ( $total > $max ) {
+            for my $n ( keys %w ) {
+                $w{$n}--;
+                $total--;
+            }
+        }
+    }
+
+    return \%w;
+}
+
 1;
 
 __END__
